@@ -1,6 +1,9 @@
+'''
+	Copyright (c) 2022 Skyflow, Inc.
+'''
 from requests.sessions import PreparedRequest
 from ._config import ConnectionConfig
-from skyflow.errors._skyflowerrors import *
+from skyflow.errors._skyflow_errors import *
 import requests
 import json
 
